
#
# ShellCommandProcessor
#
# Processes the input and output of shell commands.
#
# Original idea by Steve Spicklemire, Silicon Prairie Ventures Inc. (SPVI)
# (Re-)Written by Philipp "philiKON" von Weitershausen
# ThreadedReader class taken from ZCVSFolder's ShellCommandProcessor
#

"""
$Id: ShellCommandProcessor.py,v 1.5 2003/03/17 13:49:49 philipp Exp $
"""

from threading import Thread

try:
    from popen2 import Popen3
    Popen3Available = 1
except ImportError:
    from popen2 import popen3
    Popen3Available = 0

from interfaces.ShellCommandProcessor import IShellCommandProcessor

class ThreadedReader(Thread):
    """
    Thread for reading a file in the background
    """
    
    def __init__(self, fd):
        Thread.__init__(self)
        self.fd = fd

    def run(self):
        self.result = self.fd.read()
        self.fd.close()

class ShellCommandProcessor:

    __implements__ = IShellCommandProcessor

    def _processUnix(self, command, pipe):
        child = Popen3(command, 1, -1)
        child_stdout, child_stdin = child.fromchild, child.tochild
        child_stderr = child.childerr

        if pipe:
            child_stdin.write(pipe)
            child_stdin.flush()
        child_stdin.close()

        # Create a thread for reading stderr in the background
        reader = ThreadedReader(child_stderr)
        reader.start()

        # Read stdout in the foreground
        result_out = child_stdout.read()
        child_stdout.close()

        # Wait for the child to finish
        reader.join()
        result_err = reader.result
        result_val = child.wait()

        return result_val, result_out, result_err

    def _processOther(self, command, pipe):
        stdout, stdin, stderr = popen3(command, -1, 'b')

        if pipe:
            stdin.write(pipe)
            stdin.flush()
        stdin.close()

        # Create a thread for reading stderr in the background
        reader = ThreadedReader(child_stderr)
        reader.start()

        # Read stdout in the foreground
        result_out = child_stdout.read()
        child_stdout.close()

        # Wait for the child to finish
        reader.join()
        result_err = reader.result
        
        # popen3 does not support result values, so just assume this:
        # philiKON: not a good idea because some programs make lot of stderr
        # output without actually failing
        #if result_err:
        #    result_val = 1
        #else:
        #    result_val = 0
        result_val = 0
    
        return result_val, result_out, result_err

    ########################################################################
    # Implementation methods for interface
    # interfaces.ShellCommandProcessor.IShellCommandProcessor

    def processStreams(self, command, pipe=None):
        if Popen3Available:
            return self._processUnix(command, pipe)
        else:
            return self._processOther(command, pipe)

    #
    ########################################################################

if __name__ == '__main__':
    cmd = ShellCommandProcessor()
    print cmd.processStreams('ls -lga')
