#
# RevisionControlled interfaces
#
# Revision controlled files and directories
# File system only!
#
# Copyright (c) 2001-2003 by Philipp "philiKON" von Weitershausen
#
# This software is distributed under the terms of the Mozilla Public License
# (MPL)
#

"""
$Id: RevisionControlled.py,v 1.2 2003/03/17 13:49:52 philipp Exp $
"""

from Interface import Interface, Attribute

class IRCFile(Interface):
    """
    Revision controlled file
    """

    filename = Attribute("""
        Filename as string.
        """)

    relative_path = Attribute("""
        Relative file path as string.
        """)

    absolute_path = Attribute("""
        Absolute file path as string.
        """)

    isDirectory = Attribute("""
        Should always be false.
        """)
    
    def __init__(module_path, relative_path, status=None):
        """
        'module_path' is the absolute file path to the module.

        'relative_path' is the relative file path in the module.

        'status' can be an optional mapping containing default status information.
        """

    def getStatus():
        """
        Return status information (directly).
        """

    def getStatusInfo(status=None):
        """
        Return status information as a mapping with pathnames as keys.

        'status' is an optional mapping to which the information is written.
        """

    def setStatusInfo(statusdict):
        """
        Set the internal status information according to the information provided
        in 'statusdict' mapping.
        """

class IRCDirectory(Interface):
    """
    Directory containing revision controlled files
    """

    filename = Attribute("""
        Directory filename as string.
        """)

    relative_path = Attribute("""
        Relative directory path as string.
        """)

    absolute_path = Attribute("""
        Absolute directory path as string.
        """)

    isDirectory = Attribute(
        """Should always be true"""
        )

    def __init__(relative_path, module, status=None):
        """
        'relative_path' is the relative file path as string.

        'module' is an object implementing IModule.
        
        'status' can be an optional mapping containing default status information.
        """

    def getStatusInfo(status=None, recursive=1):
        """
        Return status information as a mapping with pathnames as keys.

        'status' is an optional mapping to which the information is written.

        If 'recursive' is true, the information of all files in all sub-
        directories is returned.
        """

    def setStatusInfo(statusdict):
        """
        Set the internal status information according to the information provided
        in 'statusdict' mapping.
        """

    def getContents():
        """
        Return a sequence of objects representing the files and
        directories in this directory.
        """

    def getDirectoryListing(recursive=0):
        """
        Return a sorted sequence with directory contents.

        If 'recursive' is true, all files in all sub-directories are returned.
        """

    def hasFile(filename):
        """
        Return true if this directory contains a file with the file name
        'filename'.
        """

    def traverse(path):
        """
        Return an object representing the file or directory identified
        by 'path'.
        
        'path' is relative to the result of getPath().
        """
