#
# IRevisionController
#
# Interface for revision controllers like CVS
#
# Copyright (c) 2001-2003 by Philipp "philiKON" von Weitershausen
#
# This software is distributed under the terms of the Mozilla Public License
# (MPL)
#

"""
$Id: RevisionController.py,v 1.2 2003/03/17 13:49:52 philipp Exp $
"""

from Interface import Interface

class IRevisionController(Interface):
    """
    This interface specifies methods for revision controllers such as a
    wrapper class to the 'cvs' program.
    """

    def initRepository():
        """
        Initialize a repository
        """

    def status(filename, recursive=1):
        """
        Return status of all objects/files specified in 'filename'. Information
        is return as a sequence of dictionaries.
	
        The parameter 'filename' is a string containing one or more filenames
        separated by white spaces. If filename is an empty string, all files
        in the module directory are taken into account.

        If 'recursive' is true, files in sub-directories are taken into account
        as well.
        """

    def checkout(filename, recursive=1, prune_empty=0):
        """
        Checkout file(s) specified by 'filename'. The parsed output is returned
        as a sequence of dictionaries.

        If 'prune_empty' is true, empty directories are pruned away.
        """

    def commit(log_message, filename="", recursive=1):
        """
        Commit file(s) specified in 'filename'.
        """

    checkin = commit

    def add(filename, binary=0):
        """
        Add file(s) specified in 'filename' to the repository.

        If 'binary' is specified, the given file(s) are treated as binary
        files, i.e. ones not containing some sort of text.
        """

    def remove(filename):
        """
        Remove file(s) specified in 'filename' from the repository
        """

    def update(filename, recursive=1, prune_empty=0, new_dirs=0):
        """
        Update file(s) specified in filename.

        If 'new_dirs' is true, directories new in the module will be created
        in the working copy.
        """

    def importModule(vendor_tag, release_tag, log_message, ignore_files=[],
                     import_dir=None):
        """
        Import files with 'vendor_tag' and 'release_tag'.
        
        'log_message' is used for the CVS log message.
        
        'ignore_files' can be an ignore pattern that is passed to CVS.

        'import_dir' can be a path to a separate import directory
        """

    def tag(tag, filename='', recursive=1):
        """
        Tag file(s) specified in 'filename' with tag.
        """

    def log(filename='', recursive=1):
        """
        Return log for file(s) specified in 'filename'.
        """

    def diff(filename='', rev1='', rev2='', recursive=1):
        """
        Return the difference between two revisions, 'rev1' and 'rev2', of all
        files specified in 'filename' as a unix diff.
        """

    def merge(rev1, rev2, filename='', recursive=1):
        """
        Merge the differences of 'filename' between 'rev1' and 'rev2' into
        the working copy.
        """

    def getLocallyModifiedFiles(filename='', recursive=1):
        """
        Return a sequence of filenames belonging to locally modified files.
        """
