
"""
$Id: testCVSObjects.py,v 1.2 2003/03/17 13:49:54 philipp Exp $
"""

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
import Products
from Products.RevisionManager.CVSObjects import \
     CVSFile, CVSDirectory, default_status
from Products.RevisionManager.interfaces.RevisionControlled import \
     IRCFile, IRCDirectory

ZopeTestCase.installProduct('RevisionManager')

TESTFILES_DIR = os.path.join(os.path.dirname(Products.RevisionManager.__file__),
                             'tests', 'testfiles')

class CVSFileTest(ZopeTestCase.ZopeTestCase):

    def afterSetUp(self):
        self.directory = CVSDirectory(TESTFILES_DIR, '')

    def afterClear(self):
        pass

    def test_iverify(self):
        """Verify interface implementation of CVSFile"""
        from Interface.Verify import verifyObject
        cvsfile = self.directory.getContents()[0]
        self.failUnless(verifyObject(IRCFile, cvsfile))
        self.failIf(cvsfile.isDirectory)

    def test_settings(self):
        """Test settings on directories"""
        helloworld = self.directory.getFileObject('helloworld.txt')
        status = helloworld.getStatus()
        status_info = helloworld.getStatusInfo()
        # test whether it returns the default status in the beginning
        self.assertEqual(status, default_status)
        # test whether getStatusInfo() and getStatus() essentially return the
        # same information
        self.assertEqual(status, status_info['helloworld.txt'])
        # set its status and compare that with what getStatusInfo() returns then
        test_status = {'filename': 'helloworld.txt',
                       'rpath':    'helloworld.txt',
                       'status':   'Up-to-date',
                       'wrev':     '1.1.1.1',
                       'rrev':     '1.1.1.1',
                       'stag':     '',
                       'sdate':    '',
                       'sopt':     '',
                       'tags':     {} }
        status_info['helloworld.txt'] = test_status
        helloworld.setStatusInfo(status_info)
        self.assertEqual(helloworld.getStatusInfo(), status_info)

class CVSDirectoryTest(ZopeTestCase.ZopeTestCase):

    def afterSetUp(self):
        self.directory = CVSDirectory(TESTFILES_DIR, '')

    def afterClear(self):
        pass

    def test_iverify(self):
        """Verify interface implementation of CVSDirectory"""
        from Interface.Verify import verifyObject
        self.failUnless(verifyObject(IRCDirectory, self.directory))
        self.failUnless(self.directory.isDirectory)

    def test_settings(self):
        """Test settings on files"""
        directory = self.directory
        status_info = directory.getStatusInfo()
        # check whether the status of all files in the directory has been returned
        files = status_info.keys()
        files.sort()
        self.assertEqual(directory.getDirectoryListing(), files)
        # set its status and compare that with what the file objects tell us then
        test_status = {'filename': 'different each time',
                       'rpath':    'different each time',
                       'status':   'Up-to-date',
                       'wrev':     '1.1.1.1',
                       'rrev':     '1.1.1.1',
                       'stag':     '',
                       'sdate':    '',
                       'sopt':     '',
                       'tags':     {} }
        for file in files:
            status_info['file'] = test_status
        directory.setStatusInfo(status_info)
        for file in files:
            fileobj = directory.getFileObject(file)
            self.assertEqual(fileobj.getStatus(), status_info[file])

    def test_has_file(self):
        """Test hasFile()"""
        directory = self.directory
        expected = os.listdir(TESTFILES_DIR)
        expected.remove("CVS")
        for filename in expected:
            self.failUnless(directory.hasFile(filename))
        self.failIf(directory.hasFile('definitelynotindirectory'))

    def test_get_file_object(self):
        """Test getFileObject()"""
        directory = self.directory
        helloworld = directory.getFileObject('helloworld.txt')
        self.failUnless(helloworld.__class__ is CVSFile)
        self.assertRaises(ValueError, directory.getFileObject, 'definitelynotindirectory')

    def test_get_contents(self):
        """Test getContents()"""
        directory = self.directory
        contents = directory.getContents()
        filenames = [obj.filename for obj in contents]
        self.assertEqual(filenames, directory.getDirectoryListing())

    def test_directory_listing(self):
        """Test directory listing"""
        expected = os.listdir(TESTFILES_DIR)
        expected.remove("CVS")
        expected.sort()
        self.assertEqual(self.directory.getDirectoryListing(1), expected)

    def test_traverse(self):
        """Test file object traversal"""
        directory = self.directory
        self.failUnless(directory.traverse('') is directory)
        self.failUnless(directory.traverse('helloworld.txt') is directory.getFileObject('helloworld.txt'))
        marker = []
        self.failUnless(directory.traverse('definitelynotindirectory', marker) is marker)
        #XXX missing traversal through subdirectories

if __name__ == '__main__':
    framework(descriptions=1, verbosity=2)
else:
    import unittest
    def test_suite():
        suite = unittest.TestSuite()
        suite.addTest(unittest.makeSuite(CVSFileTest))
        suite.addTest(unittest.makeSuite(CVSDirectoryTest))
        return suite

